# -*- coding: utf-8 -*-
from physics import PhysicsEngine
import numpy as np

class PhysicsEngineRK4(PhysicsEngine):
    
    dt = None

    def __init__(self, world, dt):
        super(PhysicsEngineRK4, self).__init__(world)
        self.dt = dt

    def calc_acceleration(self, n, positions, masses):
        accelerations = np.array([np.zeros(3) for i in range(n)])
        #force loop
        for i in range(n):
            for j in range(i + 1, n):
                force = self.grav_force(
                        positions[i], 
                        positions[j], 
                        masses[i], 
                        masses[j])
                accelerations[i] += force / masses[i]
                accelerations[j] -= force / masses[j]
        return accelerations
    
    def step(self):
        world_objects = list(self.world.world_objects.values())
        n = len(world_objects)
        
        positions = np.array([wo.position for wo in world_objects])
        velocities = np.array([wo.velocity for wo in world_objects])
        masses = np.array([wo.mass for wo in world_objects])
        
        k1_v = self.calc_acceleration(n, positions, masses)
        k1_r = velocities
        k2_v = self.calc_acceleration(n, positions + k1_r * self.dt / 2, masses)
        k2_r = velocities + k1_v * self.dt / 2
        k3_v = self.calc_acceleration(n, positions + k2_r * self.dt / 2, masses)
        k3_r = velocities + k2_v * self.dt / 2
        k4_v = self.calc_acceleration(n, positions + k3_r * self.dt, masses)
        k4_r = velocities + k3_v * self.dt
        
        velocities = velocities + self.dt / 6 * (k1_v + 2*k2_v + 2*k3_v + k4_v)        
        positions = positions + self.dt / 6 * (k1_r + 2*k2_r + 2*k3_r + k4_r)
        
        for i in range(n):
            wo = world_objects[i]
            wo.position = positions[i]
            wo.velocity = velocities[i]
        
        self.world.time += self.dt